# ==================================================================
# Links SQL - enhanced directory management system
#
#   Website  : http://gossamer-threads.com/
#   Support  : http://gossamer-threads.com/scripts/support/
#   CVS Info : 087,070,083,092,087 
#   Revision : $Id: Favorites.pm,v 1.2 2001/11/28 16:04:26 paul Exp $
#
# Copyright (c) 2001 Gossamer Threads Inc.  All Rights Reserved.
# Redistribution in part or in whole strictly prohibited. Please
# see LICENSE file for full details.
# ==================================================================

package Links::User::Favorites;
# ==================================================================
    use strict;
    use Links qw/$DB $IN $USER $CFG/;
    use Links::SiteHTML;
    use Links::Plugins;

sub handle {
# --------------------------------------------------------------
# Handle the request.
#

    if    (($IN->param('do') eq 'add') and  ($IN->param('id') =~ /^\d+$/))    {   _add_fave();     }
    elsif (($IN->param('do') eq 'delete') and ($IN->param('id') =~ /^\d+$/))  {   _delete_fave();  }
    elsif ($IN->param('do') eq 'delete_all')                                  {   _delete_all();   }
    else                                                                      {   _view_fave();    }

}

sub _add_fave {
# --------------------------------------------------------------
# Add a favorite to the cookie
#

# Retrieve the user options
    my $opts     = Links::Plugins->get_plugin_user_cfg ('MyLinks_SQL');
    my ($new_cookie,$cookie);
    my ($id)     = $IN->param('id');
    my (@ids)    = split /-/, $IN->cookie($opts->{'Cookie Name'});
    my ($total)  = scalar @ids;
    my ($db)     = $DB->table('Links');
    my (@result) = $db->select ('ID', { ID => $id })->fetchrow_array;
    
# Spew an error if there is no such link in the database
    if (!$result[0]) {
        print $IN->header();
	  print Links::SiteHTML::display ('fave_error', { error => $opts->{'No Such Link'} } );
	  return;
    }

# If the save limit has been reached, show an error.
    if ($total == $opts->{'Max Per User'}) {
        print $IN->header();
	  print Links::SiteHTML::display ('fave_error', { error => $opts->{'Save Max'} } );
	  return;
    }

# No duplicates allowed
    if ($#ids > -1) {
        if (grep { /^$id$/ } @ids) {		    
	      print $IN->header();
	      print Links::SiteHTML::display ('fave_error', { error => $opts->{'Duplicate Error'} } );
	      return;
	}

    }

    push @ids, $id;
    $new_cookie = join '-', @ids;

    $cookie = $IN->cookie( -name    => $opts->{'Cookie Name'},
			         -value   => $new_cookie,
			         -expires => '+1yr',
			         -domain  => '' );

# Set the new cookie
    print $IN->header( -cookie => $cookie );
    print Links::SiteHTML::display ('fave_added');
    return;

}

sub _delete_fave {
# --------------------------------------------------------------
# Delete a favorite
#

# Retrieve the user options again.
    my $opts     = Links::Plugins->get_plugin_user_cfg ('MyLinks_SQL');
    my ($new_cookie,$cookie);
    my ($id) = $IN->param('id');
    my (@ids) = split /-/, $IN->cookie($opts->{'Cookie Name'});
 
# If the id exists in the cookie delete it otherwise show an error
    if (grep { /^$id$/ } @ids) {
        $new_cookie = join('-', grep { !/^$id$/ } @ids);
    }
    else {
	print $IN->header();
	print Links::SiteHTML::display ('fave_error', { error => $opts->{'Delete Non-Existant'} } );
	return;
    }

    $cookie = $IN->cookie( -name    => $opts->{'Cookie Name'},
			         -value   => $new_cookie,
			         -expires => '+1yr',
			         -domain  => '' );

    print $IN->header( -cookie => $cookie );
    print Links::SiteHTML::display ('fave_deleted');
    return;

}
	

sub _view_fave {
# --------------------------------------------------------------
# Show the main favorites page.
#
    my $opts     = Links::Plugins->get_plugin_user_cfg ('MyLinks_SQL');
    my ($link,$output,$begin,$end,$toolbar);
    my ($i) = 1;
    my ($per_page) = $opts->{'Max Per Page'};
    my ($nh) = $IN->param('nh');
    my ($db)    = $DB->table('Links');
    my (@ids)   = split /-/, $IN->cookie($opts->{'Cookie Name'});
    my ($total) = scalar @ids;

# This needs improvement but hey, it works.
    if ($total) {

        my %ids = map { $i++ => $_ } @ids;

	if ($nh > 1) {
	    $begin = ($nh * $per_page) - ($per_page - 1);
	    $end   = ($nh * $per_page);
	}
	else {
	    $begin = 1;
	    $end   = $per_page;
	}

        for ($begin .. $end) {
	       last if not exists $ids{$_};
             $link = $db->select ('*', { ID => $ids{$_} })->fetchrow_hashref;
             $output .= Links::SiteHTML::display ('fave_link', $link);
	}

    }

# Very cool toolbar routine.
    $toolbar = build_toolbar ( { url => 'fave.cgi?', numlinks => $total, nh => $nh || 1, mh => $per_page } );

    print $IN->header();
    print Links::SiteHTML::display ('fave_main', { NEXT => $toolbar, LINKS => $output, TOTAL => $total } );

}

sub _delete_all {
# --------------------------------------------------------------
# Trash the cookie.
#
    my $opts     = Links::Plugins->get_plugin_user_cfg ('MyLinks_SQL');
    my ($cookie);
    my (@ids)   = split /-/, $IN->cookie($opts->{'Cookie Name'});
    my ($total) = scalar @ids;

# No saved links?...show an error
    if (!$total) {
	print $IN->header();
	print Links::SiteHTML::display ('fave_error', { error => $opts->{'Empty Error'} } );
	return;
    }

    $cookie = $IN->cookie( -name    => $opts->{'Cookie Name'},
			         -value   => '',
			         -expires => '+1yr',
			         -domain  => '' );

    print $IN->header( -cookie => $cookie );
    print Links::SiteHTML::display ('fave_deleted_all');
    return;

}

sub build_toolbar {
# --------------------------------------------------------
# Create an Altavista style toolbar for the next and previous pages.
#
    my $opts = shift;

    my $root_url = $opts->{url};
    my $numhits  = $opts->{numlinks};
    my $nh       = $opts->{nh};
    my $mh       = $opts->{mh};

    my ($next_hit, $prev_hit, $maxhits, $left, $right, $upper, $lower, $url, $i);

    $maxhits  = $mh || $CFG->{user_save_fave_per_page};
    $next_hit = $nh + 1; 
    $prev_hit = $nh - 1;

# First, set how many pages we have on the left and the right.
    $left  = $nh; $right = int($numhits/$maxhits) - $nh;
# Then work out what page number we can go above and below.
    ($left > 7)  ? ($lower = $left - 7) : ($lower = 1);
    ($right > 7) ? ($upper = $nh + 7)   : ($upper = int($numhits/$maxhits) + 1);
# Finally, adjust those page numbers if we are near an endpoint.
    (7 - $nh >= 0) and ($upper = $upper + (8 - $nh));
    ($nh > ($numhits/$maxhits - 7)) and ($lower = $lower - ($nh - int($numhits/$maxhits - 7) - 1));
    $url = "";
# Then let's go through the pages and build the HTML.
    ($nh > 1) and ($url .= qq~<a href="$root_url&nh=$prev_hit">[<<]</a> ~);
    for ($i = 1; $i <= int($numhits/$maxhits) + ( $numhits % $maxhits ? 1 : 0 ); $i++) {
        if ($i < $lower) { $url .= " ... "; $i = ($lower-1); next; }
        if ($i > $upper) { $url .= " ... "; last; }
        ($i == $nh) ? ($url .= qq~$i ~) : ($url .= qq~<a href="$root_url&nh=$i">$i</a> ~);
        if ($i * $maxhits == $numhits) { $nh == $i and $next_hit = $i; last; }
    }
    $url .= qq~<a href="$root_url&nh=$next_hit">[>>]</a> ~ unless ($next_hit == $nh or ($nh * $maxhits > $numhits));
    return $url;
}


1;